@@ -1,5 +1,7 @@
 '''Tests the "helloworld" example.'''
 from pulsar import send, SERVER_SOFTWARE, HttpClient, get_application
+from pulsar import MultiDeferred
+from pulsar.utils.httpurl import range
 from pulsar.apps.test import unittest, run_on_arbiter, dont_run_with_thread
 
 from .manage import server
@@ -15,8 +17,9 @@ class TestHelloWorldThread(unittest.TestCase):
     
     @classmethod
     def setUpClass(cls):
-        s = server(bind='127.0.0.1:0', name=cls.name(),
-                   concurrency=cls.concurrency)
+        name = name=cls.name()
+        kwargs = {'%s__bind' % name: '127.0.0.1:0'}
+        s = server(name=cls.name(), concurrency=cls.concurrency, **kwargs)
         outcome = send('arbiter', 'run', s)
         yield outcome
         cls.app = outcome.result
@@ -25,7 +28,7 @@ class TestHelloWorldThread(unittest.TestCase):
     @classmethod
     def tearDownClass(cls):
         if cls.app is not None:
-            outcome = send('arbiter', 'kill_actor', cls.app.mid)
+            outcome = send('arbiter', 'kill_actor', cls.app.name)
             yield outcome
     
     @run_on_arbiter
@@ -33,6 +36,9 @@ class TestHelloWorldThread(unittest.TestCase):
         app = get_application(self.name())
         self.assertEqual(app.name, self.name())
         self.assertTrue(app.monitor.running())
+        self.assertEqual(app, app.app)
+        self.assertEqual(str(app), app.name)
+        self.assertEqual(app.cfg.bind, '127.0.0.1:0')
         
     def testResponse(self):
         c = HttpClient()
@@ -46,6 +52,11 @@ class TestHelloWorldThread(unittest.TestCase):
         self.assertTrue(headers)
         self.assertEqual(headers['content-type'], 'text/plain')
         self.assertEqual(headers['server'], SERVER_SOFTWARE)
+        
+    def test_getbench(self):
+        c = HttpClient()
+        yield MultiDeferred((c.get(self.uri) for _ in range(1))).lock()
+    test_getbench.__benchmark__ = True
 
 
 @dont_run_with_thread
