@@ -7,46 +7,42 @@ import pulsar
 from pulsar import Actor, make_async, safe_async, is_failure, HaltServer,\
                      Monitor, loop_timeout, Deferred, get_actor, async
 from pulsar.async.defer import pickle
-from pulsar.utils import event
+from pulsar.utils import events
 from pulsar.utils.log import LogInformation
 
 __all__ = ['Application',
            'MultiApp',
-           'ApplicationHandlerMixin',
            'Worker',
            'ApplicationMonitor',
            'get_application']
 
+APP_EVENTS = ('start', 'stop')
 
-class safe_monitor:
-    '''Decorator for monitor and application functions'''    
-    def __init__(self, event=None):
-        self.event = event
+def app_events(app):
+    for name in APP_EVENTS:
+        yield name, Deferred(description='%s.%s' % (app.name, name))
+        
+def fire_event(f):
+    name = f.__name__[3:]
+    if name not in APP_EVENTS:
+        raise ValueError()
     
-    def _halt(self, failure, app):
-        failure.log()
-        if app.can_kill_arbiter:
-            raise HaltServer('Unhandled exception application.')
+    def fire(result, app):
+        if not is_failure(result):
+            result = app
         else:
-            return failure
-
-    def signal_event(self, result, app):
-        if self.event:
-            app.events[self.event].callback(result)
-        return result
+            result.log()
+        events.fire(name, app)
+        app.events[name].callback(app)
+        
+    def _(*args, **kwargs):
+        app = args[0].params.app
+        safe_async(f, args, kwargs).addBoth(lambda r: fire(r, app))
     
-    def __call__(self, f):
-        def _(*args, **kwargs):
-            app = args[0].app
-            result = safe_async(f, args, kwargs)
-            res = result.add_callback(lambda r: app,
-                                      lambda f: self._halt(f, app))
-            return res.addBoth(lambda r: self.signal_event(r, app))
-        _.__name__ = f.__name__
-        _.__doc__ = f.__doc__
-        return _
-
-
+    _.__name__ = f.__name__
+    _.__doc__ = f.__doc__
+    return _
+            
 def get_application(name):
     '''Invoked in the arbiter domain, this function will return
 the :class:`Application` associated with *name* if available. If not in the
@@ -56,33 +52,11 @@ the :class:`Application` associated with *name* if available. If not in the
         monitor = actor.monitors.get(name)
         if monitor:
             return getattr(monitor, 'app', None) 
-    
-    
-class ApplicationHandlerMixin(object):
-    '''A mixin for both :class:`Worker` and :class:`ApplicationMonitor`.
-It implements the :meth:`handle_request` actor method
-used for by the :class:`Application` for handling requests and
-sending back responses.
-'''
-    def on_event(self, fd, event):
-        '''Override :meth:`pulsar.Actor.on_event` to delegate handling
-to the underlying :class:`Application`.'''
-        return self.app.on_event(self, fd, event)
-
-    def handle_task(self):
-        if self.information.log():
-            self.logger.info('Processed %s requests', self.info.nr)
-        try:
-            self.cfg.worker_task(self)
-        except:
-            pass
-        self.app.worker_task(self)
 
 
-class Worker(ApplicationHandlerMixin, Actor):
+class Worker(Actor):
     """\
 An :class:`Actor` class for serving an :class:`Application`.
-It provides two new methods inherited from :class:`ApplicationHandlerMixin`.
 
 .. attribute:: app
 
@@ -97,25 +71,20 @@ It provides two new methods inherited from :class:`ApplicationHandlerMixin`.
     The application handler obtained from :meth:`Application.handler`.
 
 """
-    def on_init(self, app=None, **kwargs):
-        self.app = app
-        self.information = LogInformation(self.cfg.logevery)
-        app.worker_init(self)
-        self.app_handler = app.handler()
-        return kwargs
+    @property
+    def app(self):
+        return self.params.app
 
     # Delegates Callbacks to the application
-
     def on_start(self):
+        app = self.app
+        self.app_handler = app.handler()
         self.app.worker_start(self)
         try:
             self.cfg.worker_start(self)
         except:
             pass
 
-    def on_task(self):
-        self.handle_task()
-
     def on_stop(self):
         return self.app.worker_stop(self)
 
@@ -128,57 +97,61 @@ It provides two new methods inherited from :class:`ApplicationHandlerMixin`.
 
     def on_info(self, info):
         return self.app.on_info(self, info)
+    
+    def on_event(self, fd, event):
+        '''Override :meth:`pulsar.Actor.on_event` to delegate handling
+to the underlying :class:`Application`.'''
+        return self.app.on_event(self, fd, event)
 
 
-class ApplicationMonitor(ApplicationHandlerMixin, Monitor):
+class ApplicationMonitor(Monitor):
     '''A specialized :class:`Monitor` implementation for managing
 pulsar subclasses of :class:`Application`.
 '''
     actor_class = Worker
-    
-    def on_init(self, app=None, **kwargs):
-        self.app = app
-        self.information = LogInformation(self.cfg.logevery)
-        app.monitor_init(self)
-        if not self.cfg.workers:
-            self.app_handler = app.handler()
-        else:
-            self.app_handler = app.monitor_handler()
-        return super(ApplicationMonitor, self).on_init(**kwargs)
 
+    @property
+    def app(self):
+        return self.params.app
+        
     ############################################################################
     # Delegates Callbacks to the application
-    
-    @safe_monitor('on_start')
+    @fire_event
     def on_start(self):
-        self.app.monitor_start(self)
-        # If no workears are available invoke the worker start method too
+        super(ApplicationMonitor, self).on_start()
+        app = self.app
+        app.monitor_start(self)
+        # If no workers available invoke the worker start method too
         if not self.cfg.workers:
-            self.app.worker_start(self)
+            self.app_handler = app.handler()
+            app.worker_start(self)
+        else:
+            self.app_handler = app.monitor_handler()
 
-    @safe_monitor()
-    def monitor_task(self):
-        yield self.app.monitor_task(self)
-        # There are no workers, the monitor do their job
-        if not self.cfg.workers:
-            yield self.handle_task()
-
-    @async()
+    @fire_event
     def on_stop(self):
+        app = self.params.app
         if not self.cfg.workers:
-            yield self.app.worker_stop(self)
-        yield self.app.monitor_stop(self)
+            yield app.worker_stop(self)
+        yield app.monitor_stop(self)
         yield super(ApplicationMonitor, self).on_stop()
-        event.fire('stop', sender=self.app)
-        self.app.events['on_stop'].callback(self.app)
 
     def on_exit(self):
-        self.app.monitor_exit(self)
+        app = self.params.app
+        app.monitor_exit(self)
         try:
             self.cfg.worker_exit(self)
         except:
             pass
-
+    
+    def monitor_task(self):
+        self.app.monitor_task(self)
+        
+    def on_event(self, fd, event):
+        '''Override :meth:`pulsar.Actor.on_event` to delegate handling
+to the underlying :class:`Application`.'''
+        return self.app.on_event(self, fd, event)
+    
     def actorparams(self):
         '''Override the :meth:`Monitor.actorparams` method to
 updated actor parameters with information about the application.
@@ -186,8 +159,8 @@ updated actor parameters with information about the application.
 :rtype: a dictionary of parameters to be passed to the
     spawn method when creating new actors.'''
         p = Monitor.actorparams(self)
-        app = self.app
-        if app.cfg.concurrency == 'thread':
+        app = self.params.app
+        if self.cfg.concurrency == 'thread':
             app = pickle.loads(pickle.dumps(app))
         p.update({'app': app,
                   'name': '{0}-worker'.format(app.name)})
@@ -195,7 +168,7 @@ updated actor parameters with information about the application.
 
     def on_info(self, info):
         if not self.cfg.workers:
-            return self.app.on_info(self, info)
+            return self.params.app.on_info(self, info)
         else:
             return info
 
@@ -266,15 +239,6 @@ These are the most important facts about a pulsar :class:`Application`
     full path of the script which starts the application or ``None``.
     If supplied it is used to setup the python path
 
-.. attribute:: can_kill_arbiter
-
-    If ``True``, an unhandled error in the application will shut down the
-    :class:`pulsar.Arbiter`. Check the :meth:`ApplicationMonitor.monitor_task`
-    method for implementation.
-
-    Default: ``False``.
-
-
 .. attribute:: commands_set
 
     Optional set of :ref:`remote actions <api-remote_commands>` available
@@ -289,7 +253,6 @@ These are the most important facts about a pulsar :class:`Application`
     cfg_apps = None
     config_options_include = None
     config_options_exclude = None
-    can_kill_arbiter = False
     commands_set = None
     monitor_class = ApplicationMonitor
 
@@ -301,7 +264,6 @@ These are the most important facts about a pulsar :class:`Application`
                  argv=None,
                  script=None,
                  version=None,
-                 can_kill_arbiter=None,
                  parse_console=True,
                  commands_set=None,
                  cfg=None,
@@ -317,8 +279,6 @@ These are the most important facts about a pulsar :class:`Application`
     only if the arbiter has not yet started.
 '''
         self.description = description or self.description
-        if can_kill_arbiter is not None:
-            self.can_kill_arbiter = bool(can_kill_arbiter)
         self.epilog = epilog or self.epilog
         self._app_name = self._app_name or self.__class__.__name__.lower()
         self._name = name or self._app_name
@@ -342,22 +302,22 @@ These are the most important facts about a pulsar :class:`Application`
             monitor = actor.monitors.get(self.name)
         if monitor is None and (not actor or actor.is_arbiter()): 
             # Add events
-            self.local.events = {'on_start': Deferred(), 'on_stop': Deferred()}
+            self.local.events = dict(app_events(self))
             self.cfg.on_start()
             self.configure_logging()
-            event.fire('ready', sender=self)
-            arbiter = pulsar.arbiter(cfg=self.cfg)
+            events.fire('ready', self)
+            arbiter = pulsar.arbiter(cfg=self.cfg.new_config())
             if self.on_config() is not False:
                 monitor = arbiter.add_monitor(self.monitor_class,
                                               self.name,
                                               app=self,
                                               cfg=self.cfg,
                                               ioqueue=self.ioqueue)
+                self.cfg = monitor.cfg
                 if self.commands_set:
                     monitor.impl.commands_set.update(self.commands_set)
-        events = self.events
-        if events:
-            return events['on_start']
+        if self.events:
+            return self.events['start']
 
     @property
     def app(self):
@@ -545,20 +505,12 @@ By default it returns ``None``.'''
 
     def on_event(self, worker, fd, events):
         pass
-
-    def worker_init(self, worker):
-        '''Callback by :class:`Worker` when initializing.'''
-        pass
     
     def worker_start(self, worker):
         '''Called by the :class:`Worker` :meth:`pulsar.Actor.on_start`
 :ref:`callback <actor-callbacks>` method.'''
         pass
 
-    def worker_task(self, worker):
-        '''Callback by the *worker* :meth:`Actor.on_task` callback.'''
-        return
-
     def worker_stop(self, worker):
         '''Called by the :class:`Worker` just after stopping.'''
         pass
@@ -573,29 +525,23 @@ By default it returns ``None``.'''
 :class:`Worker` is spawned'''
         return params
 
-    def monitor_init(self, monitor):
-        '''Callback by :class:`ApplicationMonitor` when initializing.
-This is a chance to setup your application before the application
-monitor is added to the arbiter.'''
-        pass
-
     def monitor_start(self, monitor):
         '''Callback by :class:`ApplicationMonitor` when starting.
 The application is now in the arbiter but has not yet started.'''
         pass
 
-    def monitor_task(self, monitor):
-        '''Callback by :class:`ApplicationMonitor` at each event loop'''
-        pass
-
     def monitor_stop(self, monitor):
-        '''Callback by :class:`ApplicationMonitor` at each event loop'''
+        '''Callback by :class:`ApplicationMonitor` when stopping'''
         pass
 
     def monitor_exit(self, monitor):
-        '''Callback by :class:`ApplicationMonitor` at each event loop'''
+        '''Callback by :class:`ApplicationMonitor` at exit'''
         pass
 
+    def monitor_task(self, monitor):
+        '''Callback by :class:`ApplicationMonitor` at each event loop'''
+        pass
+    
     def start(self):
         '''Start the application if it wasn't already started.'''
         arbiter = pulsar.arbiter()
