@@ -5,9 +5,10 @@ import logging
 from pulsar import to_bytes, is_failure, log_failure, is_async,\
                     as_failure, maybe_async, HttpException
 from pulsar.utils.tools import checkarity
+from pulsar.utils.structures import AttributeDictionary
 from pulsar.apps.wsgi import WsgiResponse, WsgiResponseGenerator
 
-from .decorators import callrpc, wrap_object_call
+from .decorators import rpcerror, wrap_object_call
 from .exceptions import *
 
 
@@ -15,35 +16,51 @@ __all__ = ['RpcHandler', 'RpcMiddleware']
 
 LOGGER = logging.getLogger('pulsar.rpc')
 
+LOGGER = logging.getLogger('pulsar.rpc')
+
+
 class RpcRequest(object):
 
-    def __init__(self, environ, handler, method, func, args,
-                kwargs, id, version):
+    def __init__(self, environ):
         self.environ = environ
+
+    def __repr__(self):
+        return self.rpc.method
+
+    def __getattr__(self, name):
+        return self.environ.get(name)
+
+    def __getitem__(self, name):
+        return self.environ[name]
+    
+    def __setitem__(self, name, value):
+        self.environ[name] = value
+
+
+class RPC:
+    
+    def __init__(self, handler, method, func, args, kwargs,
+                 id=None, version=None):
         self.handler = handler
         self.method = method
         self.func = func
         self.args = args
         self.kwargs = kwargs
-        self.version = version
         self.id = id
-
-    def __repr__(self):
-        return self.method
-
-    @property
-    def user(self):
-        return self.environ.get('user')
-
-    @property
-    def content_type(self):
-        return self.handler.content_type
-
-    def process(self):
-        if not self.func:
+        self.version = version
+        
+    def process(self, request):
+        func = self.func
+        if not func:
             raise NoSuchFunction('Function "%s" not available.' % self.method)
-        return callrpc(self.func, self.handler, self, self.args, self.kwargs)
-
+        try:
+            return func(self.handler, request, *self.args, **self.kwargs)
+        except TypeError as e:
+            if not getattr(func, 'FromApi', False):
+                rpcerror(func, self.args, self.kwargs, discount=2)
+            else:
+                raise
+    
 
 class ResponseGenerator(WsgiResponseGenerator):
     '''Asynchronous response generator invoked by the djpcms WSGI middleware'''
@@ -53,12 +70,13 @@ class ResponseGenerator(WsgiResponseGenerator):
 
     def __iter__(self):
         request = self.request
+        rpc = request['rpc']
         status_code = 200
         try:
-            result = request.process()
+            result = rpc.process(request)
         except Exception as e:
             result = as_failure(e)
-        handler = request.handler
+        handler = rpc.handler
         result = maybe_async(result)
         while is_async(result):
             yield b''
@@ -68,22 +86,16 @@ class ResponseGenerator(WsgiResponseGenerator):
                 e = result.trace[1]
                 status_code = getattr(e, 'status', 400)
                 log_failure(result)
-                result = handler.dumps(request.id,
-                                       request.version,
-                                       error=e)
+                result = handler.dumps(rpc.id, rpc.version, error=e)
             else:
-                result = handler.dumps(request.id,
-                                       request.version,
-                                       result=result)
+                result = handler.dumps(rpc.id, rpc.version, result=result)
         except Exception as e:
             LOGGER.error('Could not serialize', exc_info=True)
             status_code = 500
-            result = handler.dumps(request.id,
-                                   request.version,
-                                   error=e)
+            result = handler.dumps(rpc.id, rpc.version, error=e)
         content = to_bytes(result)
         response = WsgiResponse(status_code, content,
-                                content_type=request.content_type)
+                                content_type=handler.content_type)
         for c in self.start(response):
             yield c
 
@@ -144,8 +156,7 @@ class RpcHandler(MetaRpcHandler('_RpcHandler', (object,), {'virtual': True})):
     charset = 'utf-8'
     _info_exceptions = (Fault,)
 
-    def __init__(self, subhandlers=None, title=None, documentation=None,
-                 **kwargs):
+    def __init__(self, subhandlers=None, title=None, documentation=None):
         self._parent = None
         self.subHandlers = {}
         self.title = title or self.__class__.__name__
@@ -230,8 +241,7 @@ for ``method``, ``kwargs`` are keyworded parameters for ``method``,
             func = handler.rpcfunctions[method_name]
         except:
             func = None
-        return RpcRequest(environ, handler, method, func, args,
-                          kwargs, id, version)
+        environ['rpc'] = RPC(handler, method, func, args, kwargs, id, version)
 
     def invokeServiceEndpoint(self, meth, args):
         return meth(*args)
@@ -272,6 +282,7 @@ class RpcMiddleware(object):
 .. _WSGI:: http://www.wsgi.org/
 '''
     methods = ('get','post','put','head','delete','trace','connect')
+    request_class = RpcRequest
 
     def __init__(self, handler, path=None, methods=None):
         self.handler = handler
@@ -298,6 +309,7 @@ class RpcMiddleware(object):
             data = environ['wsgi.input'].read()
             hnd = self.handler
             method, args, kwargs, id, version = hnd.get_method_and_args(data)
-            request = hnd.request(environ, method, args, kwargs, id, version)
+            hnd.request(environ, method, args, kwargs, id, version)
+            request = self.request_class(environ)
             return ResponseGenerator(request, start_response)
 
