@@ -1,5 +1,18 @@
 '''An application for asynchronous applications serving requests
-on a socket.'''
+on a socket.
+An application for asynchronous applications serving requests
+on a socket. This is the base class of :class:`pulsar.apps.wsgi.WSGIServer`.
+
+.. attribute:: socket_server_class
+
+    Class or callable which returns the asynchronous socket server, usually
+    a subclass of :class:`pulsar.AsyncSocketServer`.
+    
+.. attribute:: address
+
+    The socket address, available once the application has started.
+    
+'''
 import pulsar
 from pulsar import AsyncIOStream
 
@@ -23,34 +36,19 @@ class Bind(SocketSetting):
         
 
 class SocketServer(pulsar.Application):
-    '''An application for asynchronous applications serving requests
-on a socket. This is the base class of :class:`pulsar.apps.wsgi.WSGIServer`.
-
-.. attribute:: socket_server_class
-
-    Class or callable which returns the asynchronous socket server, usually
-    a subclass of :class:`pulsar.AsyncSocketServer`.
-    
-.. attribute:: address
-
-    The socket address, available once the application has started.
-    
-'''
     _app_name = 'socket'
     socket_server_class = None
     address = None
     
-    def monitor_init(self, monitor):
+    def monitor_start(self, monitor):
         # if the platform does not support multiprocessing sockets set
         # the number of workers to 0.
         cfg = self.cfg
-        if not pulsar.platform.multiProcessSocket()\
-            or cfg.concurrency == 'thread':
-            cfg.set('workers', 0)
         if not self.socket_server_class:
             raise TypeError('Socket server class not specified.')
-        
-    def monitor_start(self, monitor):
+        if not pulsar.platform.has_multiProcessSocket\
+            or cfg.concurrency == 'thread':
+            cfg.set('workers', 0)
         # Open the socket and bind to address
         address = self.cfg.address
         if address:
@@ -67,16 +65,18 @@ on a socket. This is the base class of :class:`pulsar.apps.wsgi.WSGIServer`.
         s = self.socket_server_class(worker, worker.params.socket)
         # We add the file descriptor handler
         s.on_connection_callbacks.append(worker.handle_fd_event)
-        worker.socket_server = s.start()
+        worker.socket_server = s
     
     def worker_stop(self, worker):
         if hasattr(worker, 'socket_server'):
-            worker.ioloop.remove_handler(worker.socket_server)
-            worker.socket_server.close()
+            # we don't shut down the socket, simply remove all active
+            # connections and othe clean up operations.
+            worker.socket_server.on_close()
         
     def on_event(self, worker, fd, events):
         connection = worker.socket_server.accept()
-        return connection.on_closed
+        if connection is not None:
+            return connection.on_closed
     
     def on_info(self, worker, data):
         server = worker.socket_server
