@@ -79,7 +79,7 @@ else:
     
 import pulsar
 from pulsar.apps import tasks
-from pulsar.utils import event
+from pulsar.utils import events
 
 from .result import *
 from .case import *
@@ -171,8 +171,6 @@ is a group of tests specified in a test class.
     _app_name = 'test'
     cfg_apps = ('cpubound',)
     commands_set = test_commands
-    config_options_exclude = ('daemon', 'user', 'group')
-    can_kill_arbiter = True
     cfg = {'loglevel': 'none',
            'timeout': 3600,
            'backlog': 1,
@@ -259,7 +257,7 @@ configuration and plugins.'''
             if tests:
                 self.logger.info('loaded %s test classes', len(tests))
                 self.runner.on_start()
-                event.fire('tests', sender=self, value=tests)
+                events.fire('tests', self, tests=tests)
                 monitor.cfg.set('workers', min(self.cfg.workers, len(tests)))
                 self._time_start = None
             else:
