@@ -2,49 +2,55 @@ import sys
 import os
 import signal
 import atexit
+import socket
 from time import time
 import random
 import threading
 from multiprocessing import current_process
+from multiprocessing.queues import Empty
 from threading import current_thread
 
+try:    #pragma nocover
+    import queue
+except ImportError: #pragma nocover
+    import Queue as queue
+ThreadQueue = queue.Queue
 
 from pulsar import AlreadyCalledError, AlreadyRegistered,\
                    ActorAlreadyStarted, LogginMixin, system, Config
 from pulsar.utils.structures import AttributeDictionary
+from pulsar.utils import events
 from .eventloop import IOLoop, setid
 from .proxy import ActorProxy, ActorMessage, get_command, get_proxy
 from .defer import make_async, is_failure, iteritems, itervalues,\
-                     pickle, safe_async, log_failure, make_async, is_async,\
-                     EXIT_EXCEPTIONS
+                     pickle, async, log_failure, is_async,\
+                     as_failure, EXIT_EXCEPTIONS
 from .mailbox import IOQueue, mailbox
 from .access import set_local_data, is_mainthread, get_actor, remove_actor
 
 
-__all__ = ['is_actor', 'send', 'Actor', 'ACTOR_STATES', 'Pulsar']
+__all__ = ['is_actor', 'send', 'Actor', 'ACTOR_STATES', 'Pulsar', 'ThreadQueue']
 
-ACTOR_STATES = AttributeDictionary(INITIAL=0X0, STARTING=0X1, RUN=0x2,
-                                   STOPPING=0x3, CLOSE=0x4, TERMINATE=0x5)
-ACTOR_STATES.DESCRIPTION = {ACTOR_STATES.INITIAL: 'initial',
-                            ACTOR_STATES.STARTING: 'starting',
+ACTOR_STATES = AttributeDictionary(INITIAL=0X0,
+                                   INACTIVE=0X1,
+                                   RUN=0x2,
+                                   STOPPING=0x3,
+                                   CLOSE=0x4,
+                                   TERMINATE=0x5)
+ACTOR_STATES.DESCRIPTION = {ACTOR_STATES.INACTIVE: 'inactive',
+                            ACTOR_STATES.INITIAL: 'initial',
                             ACTOR_STATES.RUN: 'running',
                             ACTOR_STATES.STOPPING: 'stopping',
                             ACTOR_STATES.CLOSE: 'closed',
                             ACTOR_STATES.TERMINATE:'terminated'}
-MINIMUM_ACTOR_TIMEOUT = 10
-DEFAULT_ACTOR_TIMEOUT = 60
-ACTOR_NOTIFY_PERIOD = 10
-# Send messages to arbiter not before the difference of
-# now and the last message time is greater than this tolerance
-# times timeout. So for a timeout of 30 seconds, the messages will
-# go after tolerance*30 seconds (18 secs for tolerance = 0.6).
+EXIT_SIGNALS = (signal.SIGINT, signal.SIGTERM, signal.SIGABRT, system.SIGQUIT)
+#
+# LOW LEVEL CONSTANTS - NO NEED TO CHANGE THOSE ###########################
+MIN_NOTIFY = 5     # DON'T NOTIFY BELOW THIS INTERVAL
+MAX_NOTIFY = 30    # NOTIFY AT LEAST AFTER THESE SECONDS
+ACTOR_TERMINATE_TIMEOUT = 2 # TIMEOUT WHEN JOINING A TERMINATING ACTOR
 ACTOR_TIMEOUT_TOLERANCE = 0.6
-# Timeout of when joining an actor which has been terminated
-ACTOR_TERMINATE_TIMEOUT = 2
 ACTOR_STOPPING_LOOPS = 5
-EMPTY_TUPLE = ()
-EMPTY_DICT = {}
-
 
 def is_actor(obj):
     return isinstance(obj, Actor)
@@ -112,38 +118,46 @@ To spawn a new actor::
 
 Here ``a`` is actually a reference to the remote actor.
 
-.. attribute:: actor_functions
+**ATTRIBUTES**
 
-    dictionary of remote functions exposed by the actor. This
-    dictionary is filled by the :class:`pulsar.ActorMetaClass` during class
-    construction.
+.. attribute:: name
 
-.. attribute:: age
+    The name of this :class:`Actor`.
+    
+.. attribute:: aid
 
-    The age of actor, used to access how long the actor has been created.
+    Unique ID for this :class:`Actor`.
+    
+.. attribute:: commands_set
+
+    Set of :ref:`command names <actor_commands>` available to this
+    :class:`Actor`.
 
 .. attribute:: ioqueue
 
-    An optional distributed queue used as IO. Check server configuration with
-    :ref:`io queue <configuration-ioqueue>` for information.
+    An optional distributed queue. If present, it signal that the :class:`Actor`
+    is a :ref:`CPU bound worker <cpubound>` receiving task requests on the
+    :attr:`ioqueue`. 
 
     Default ``None``.
 
 .. attribute:: cpubound
 
-    Indicates if the :class:`Actor` is a CPU-bound worker or a I/O-bound one.
-    CPU-bound actors have a sperate event loop for handling I/O events.
-
-.. attribute:: ioloop
-
-    An instance of :class:`IOLoop`, the input/output event loop
-    driving the actor. Some actors may share the ioloop with other actors
-    depending on their concurrency implementation.
+    Indicates if the :class:`Actor` is a :ref:`CPU-bound worker <cpubound>`
+    or a I/O-bound one.
+    CPU-bound actors have a separate event loop for handling I/O events.
     
+    See also: :attr:`ioqueue`
+
 .. attribute:: requestloop
 
-    The :class:`IOLoop` to listen for requests. This is different from
-    :attr:`ioloop` for CPU-bound actors.
+    The :class:`IOLoop` to listen for requests.
+    
+.. attribute:: ioloop
+
+    An instance of :class:`IOLoop` which listen for input/output events
+    on a socket.  This is different from the :attr:`requestloop` only
+    for :ref:`CPU-bound actors <cpubound>`.
 
 .. attribute:: request_processed
 
@@ -154,30 +168,38 @@ Here ``a`` is actually a reference to the remote actor.
     The current number of concurrent requests the actor is serving.
     Depending on the actor type, this number can be very high or max 1
     (CPU bound actors).
+    
+.. attribute:: proxy
 
-.. attribute:: loglevel
+    Instance of a :class:`ActorProxy` holding a reference
+    to this :class:`Actor`. The proxy is a lightweight representation
+    of the actor which can be shared across different processes
+    (i.e. it is pickable).
 
-    String indicating the logging level for the actor.
+.. attribute:: linked_actors
+
+    Dictionary of :class:`ActorProxy` linked with this :class:`Actor`.
+
+.. attribute:: params
+
+    Contains parameters which are passed to actors spawned by this actor.
+    
+**METHODS**
 '''
     exit_code = None
     mailbox = None
-    stopping_start = None
-    stopping_end = None
+    signal_queue = None
     
     def __init__(self, impl):
         self.__impl = impl
-        on_task = impl.params.pop('on_task', None)
         on_event = impl.params.pop('on_event', None)
         ioqueue = impl.params.pop('ioqueue', None)
         if not self.is_arbiter():
-            if on_task:
-                self.on_task = lambda : on_task(self)
             if on_event:
                 self.on_event = lambda fd, event: on_event(self, fd, event)
         else:
             ioqueue = None
         self.request_processed = 0
-        self.last_notified = 0
         self.concurrent_requests = 0
         self.state = ACTOR_STATES.INITIAL
         self.ioqueue = ioqueue
@@ -186,8 +208,9 @@ Here ``a`` is actually a reference to the remote actor.
         self.arbiter = impl.params.pop('arbiter', None)
         self.monitor = impl.params.pop('monitor', None)
         self.proxy_mailboxes = {}
-        self.params = AttributeDictionary(self.on_init(**impl.params))
+        self.params = AttributeDictionary(**impl.params)
         del impl.params
+        setid(self)
 
     def __repr__(self):
         return self.impl.unique_name
@@ -218,9 +241,6 @@ Here ``a`` is actually a reference to the remote actor.
     
     @property
     def proxy(self):
-        '''Instance of a :class:`ActorProxy` holding a reference
-to the actor. The proxy is a lightweight representation of the actor
-which can be shared across different processes (i.e. it is pickable).'''
         return ActorProxy(self)
 
     @property
@@ -229,7 +249,6 @@ which can be shared across different processes (i.e. it is pickable).'''
 
     @property
     def ioloop(self):
-        '''The :class:`IOLoop` for I/O.'''
         return self.mailbox.ioloop
 
     @property
@@ -248,6 +267,19 @@ which can be shared across different processes (i.e. it is pickable).'''
     ############################################################################
     ##    HIGH LEVEL API METHODS
     ############################################################################
+    def start(self):
+        '''Called after forking to start the actor's life. This is where
+logging is configured, the :attr:`Actor.mailbox` is registered and the
+:attr:`Actor.ioloop` is initialised and started.'''
+        if self.state == ACTOR_STATES.INITIAL:
+            self.configure_logging()
+            self._setup_ioloop()
+            events.fire('start', self)
+            self.on_start()
+            self.periodic_task()
+            self.state = ACTOR_STATES.RUN
+            self._run()
+            
     def command(self, action):
         '''Fetch the pulsar command for *action*.'''
         return get_command(action, self.commands_set)
@@ -262,7 +294,6 @@ parameters *params*. It return a :class:`ActorMessage`.'''
     def put(self, request):
         '''Put a *request* into the :attr:`ioqueue` if available.'''
         if self.ioqueue:
-            self.logger.debug('Putting %s into IO queue', request)
             self.ioqueue.put(('request', request))
         else:
             self.logger.error("Trying to put a request on task queue,\
@@ -272,16 +303,17 @@ parameters *params*. It return a :class:`ActorMessage`.'''
     def running(self):
         '''``True`` if actor is running.'''
         return self.state == ACTOR_STATES.RUN
-
+    
+    def active(self):
+        '''``True`` if actor is active by being both running and having
+the :attr:`ioloop` running.'''
+        return self.running() and self.arbiter.aid in self.linked_actors
+    
     def started(self):
         '''``True`` if actor has started. It does not necessarily
 mean it is running.'''
         return self.state >= ACTOR_STATES.RUN
 
-    def stopping(self):
-        '''``True`` if actor is stopping.'''
-        return self.state == ACTOR_STATES.STOPPING
-
     def closed(self):
         '''``True`` if actor has exited in an clean fashion.'''
         return self.state == ACTOR_STATES.CLOSE
@@ -298,18 +330,18 @@ mean it is running.'''
         '''Return ``True`` if ``self`` is a :class:`Monitor`.'''
         return False
 
-    def isprocess(self):
+    def is_process(self):
         '''boolean indicating if this is an actor on a child process.'''
         return self.impl.kind == 'process'
 
-    def ready(self):
-        return self.arbiter.aid in self.linked_actors
-
     def can_poll(self):
-        '''Check if the actor can poll requests. This is used by CPU-bound
- actors only.'''
-        m = self.cfg.backlog
-        return self.concurrent_requests < m if m else True
+        '''Check if the actor can poll requests. This is used by 
+:ref:`CPU-bound actors <cpubound>` only.'''
+        if self.active():
+            m = self.cfg.backlog
+            return self.concurrent_requests < m if m else True
+        else:
+            return False
 
     def __reduce__(self):
         raise pickle.PicklingError('{0} - Cannot pickle Actor instances'\
@@ -318,46 +350,39 @@ mean it is running.'''
     ############################################################################
     ##    EVENT HANDLING
     ############################################################################
+    @async()
     def handle_fd_event(self, fd, event):
         '''This function should be used when registering events
- on file descriptors. It is called by the :attr:`requestloop` when an event
- occurs on file descriptor *fd*.'''
+ on file descriptors registered with the :attr:`requestloop`.'''
         self.request_processed += 1
         self.concurrent_requests += 1
-        msg = safe_async(self.on_event, args=(fd, event))
-        msg.addBoth(self.end_event)
-
-    def end_event(self, result):
+        try:
+            future = make_async(self.on_event(fd, event))
+        except Exception as e:
+            result = as_failure(e)
+        else:
+            yield future
+            result = future.result
         self.concurrent_requests -= 1
         log_failure(result)
         max_requests = self.cfg.max_requests
-        should_stop = max_requests and self.request_processed >= max_requests
-        if should_stop:
-            self.logger.info("Shutting down %s. Max requests reached.", self)
+        if max_requests and self.request_processed >= max_requests:
+            self.logger.warn("Shutting down %s. Max requests reached.", self)
             self.stop()
 
     ############################################################################
-    ##    HOOKS
+    ##    CALLBACKS
     ############################################################################
-    def on_init(self, **kwargs):
-        '''The :ref:`actor callback <actor-callbacks>` run once at the
-end of initialisation (after forking).'''
-        return kwargs
-
     def on_start(self):
-        '''The :ref:`actor callback <actor-callbacks>` run once just before
+        '''The :ref:`actor callback <actor-callbacks>` run **once** just before
 the actor starts (after forking) its event loop. Every attribute is available,
-therefore this is a chance to setup to perform custom initialization
+therefore this is a chance to setup to perform custom initialisation
 before the actor starts running.'''
         pass
 
-    def on_task(self):
-        '''The :ref:`actor callback <actor-callbacks>` executed at each
-iteration of the :attr:`Actor.ioloop`.'''
-        pass
-
     def on_event(self, fd, event):
-        '''handle and event on a filedescriptor *fd*.'''
+        '''Handle an event on a file descriptor *fd*. This is what defines the
+life of an actor.'''
         pass
 
     def on_stop(self):
@@ -381,25 +406,6 @@ iteration of the :attr:`Actor.ioloop`.'''
  :rtype: a dictionary of pickable data.'''
         return data
 
-    def start(self):
-        '''Called after forking to start the actor's life. This is where
-logging is configured, the :attr:`Actor.mailbox` is registered and the
-:attr:`Actor.ioloop` is initialised and started.'''
-        if self.state == ACTOR_STATES.INITIAL:
-            self.state = ACTOR_STATES.STARTING
-            self.configure_logging()
-            self._run()
-
-    ############################################################################
-    ##    INTERNALS
-    ############################################################################
-    def proxy_mailbox(self, address):
-        m = self.proxy_mailboxes.get(address)
-        if not m:
-            m = mailbox(address=address)
-            self.proxy_mailboxes[address] = m
-        return m
-
     ############################################################################
     # STOPPING
     ############################################################################
@@ -407,8 +413,8 @@ logging is configured, the :attr:`Actor.mailbox` is registered and the
         '''Stop the actor by stopping its :attr:`Actor.requestloop`
 and closing its :attr:`Actor.mailbox`. Once everything is closed
 properly this actor will go out of scope.'''
-        if force or self.state in (ACTOR_STATES.STARTING, ACTOR_STATES.RUN):
-            self.stopping_start = time()
+        if force or self.state <= ACTOR_STATES.RUN:
+            events.fire('stop', self)
             self.state = ACTOR_STATES.STOPPING
             self.exit_code = exit_code
             try:
@@ -427,11 +433,35 @@ properly this actor will go out of scope.'''
             if not self.is_monitor():
                 self.requestloop.stop()
             self.state = ACTOR_STATES.CLOSE
-            self.stopping_end = time()
             self.logger.debug('%s exited', self)
             remove_actor(self)
+            events.fire('exit', self)
             self.on_exit()
 
+    ############################################################################
+    #    INTERNALS
+    ############################################################################
+    def periodic_task(self):
+        if self.can_continue():
+            if self.active():
+                self.send('arbiter', 'notify', self.info())
+                secs = max(ACTOR_TIMEOUT_TOLERANCE*self.cfg.timeout, MIN_NOTIFY)
+                next = time() + min(secs, MAX_NOTIFY)
+                self.ioloop.add_timeout(next, self.periodic_task)
+            else:
+                self.ioloop.add_callback(self.periodic_task, False)
+        
+    def proxy_mailbox(self, address):
+        m = self.proxy_mailboxes.get(address)
+        if not m:
+            try:
+                m = mailbox(address=address)
+            except socket.error:
+                m = None
+            else:
+                self.proxy_mailboxes[address] = m
+        return m
+    
     def get_actor(self, aid):
         '''Given an actor unique id return the actor proxy.'''
         if aid == self.aid:
@@ -447,32 +477,12 @@ properly this actor will go out of scope.'''
         else:
             return self.monitors.get(aid)
 
-    def __call__(self):
-        '''Called in the main eventloop to perform the following
-actions:
-
-* it notifies linked actors if required (arbiter only for now)
-* it executes the :meth:`Actor.on_task` callback.
-'''
-        if self.running() and self.arbiter.aid in self.linked_actors:
-            nt = time()
-            if hasattr(self, 'last_notified'):
-                timeout = self.cfg.timeout
-                tole = min(ACTOR_TIMEOUT_TOLERANCE*timeout, ACTOR_NOTIFY_PERIOD)
-                if nt - self.last_notified < tole:
-                    nt = None
-            if nt:
-                self.last_notified = nt
-                info = self.info()
-                self.send('arbiter', 'notify', info)
-            self.on_task()
-
     def info(self):
         '''return A dictionary of information related to the actor
 status and performance.'''
         if not self.started():
             return
-        isp = self.isprocess()
+        isp = self.is_process()
         requestloop  = self.requestloop
         actor = {'name': self.name,
                  'state': self.info_state,
@@ -492,20 +502,15 @@ status and performance.'''
         if isp:
             data['system'] = system.system_info(self.pid)
         return self.on_info(data)
-
-    ############################################################################
-    #    INTERNALS
-    ############################################################################
+    
     def link_actor(self, proxy, address=None):
-        '''Add the *proxy* to the :attr:`` dictionary.
+        '''Add the *proxy* to the :attr:`linked_actors` dictionary.
 if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
         if address:
             proxy.address = address
         if not proxy.address:
             raise ValueError('Linking with a actor without address')
         self.linked_actors[proxy.aid] = proxy
-        if proxy.aid == self.arbiter.aid:
-            self.requestloop.ready = True
         # If the proxy is the actor monitor, add the arbiter
         # if the monitor is not the arbiter itself.
         # This last check is crucial in order to recursive call
@@ -514,58 +519,62 @@ if *proxy* is not a class:`ActorProxy` instance raise an exception.'''
             self.link_actor(self.arbiter, address)
         return proxy
 
-    def _init_runner(self):
-        '''Initialise the runner.'''
-        if not self.isprocess():
-            return
-        random.seed()
-        proc_name = "%s-%s" % (self.cfg.proc_name, self)
-        if system.set_proctitle(proc_name):
-            self.logger.debug('Set process title to %s', proc_name)
-        #system.set_owner_process(cfg.uid, cfg.gid)
-        if is_mainthread():
-            self.logger.debug('Installing signals')
-            # The default signal handling function in signal
-            sfun = getattr(self, 'signal', None)
-            for name in system.ALL_SIGNALS:
-                func = sfun
-                if not func:
-                    func = getattr(self,'handle_{0}'.format(name.lower()),None)
-                if func:
-                    sig = getattr(signal,'SIG{0}'.format(name))
-                    try:
-                        signal.signal(sig, func)
-                    except ValueError:
-                        break
-            #atexit.register(self.stop)
-
-    def _on_run(self):
-        '''Internal function called at the start of the actor. It builds the
-event loop which will consume events on file descriptors.'''
+    def _setup_ioloop(self):
+        # Internal function called at the start of the actor. It builds the
+        # event loop which will consume events on file descriptors
         # Inject self as the actor of this thread
         ioq = self.ioqueue
         self.requestloop = IOLoop(io=IOQueue(ioq, self) if ioq else None,
                                   poll_timeout=self.params.poll_timeout,
-                                  logger=self.logger,
-                                  ready=not self.cpubound)
+                                  logger=self.logger)
         # If CPU bound add the request handler to the request loop
         if self.cpubound:
             self.requestloop.add_handler('request',
                                           self.handle_fd_event,
                                           self.requestloop.READ)
-        self._init_runner()
+        if self.is_process():
+            random.seed()
+            proc_name = "%s-%s" % (self.cfg.proc_name, self)
+            if system.set_proctitle(proc_name):
+                self.logger.debug('Set process title to %s', proc_name)
+            #system.set_owner_process(cfg.uid, cfg.gid)
+            if is_mainthread():
+                self.logger.debug('Installing signals')
+                self.signal_queue = ThreadQueue()
+                for name in system.ALL_SIGNALS:
+                    sig = getattr(signal, 'SIG%s' % name)
+                    try:
+                        signal.signal(sig, self._queue_signal)
+                    except ValueError:
+                        break
         self.mailbox = mailbox(self)
         set_local_data(self)
-        setid(self)
-        self.state = ACTOR_STATES.RUN
-        self.on_start()
-        self.logger.info('address %s', self.mailbox.address)
+        self.logger.info('%s started at address %s', self, self.mailbox.address)
 
+    def _queue_signal(self, sig, frame=None):
+        self.signal_queue.put(sig)
+        
+    def can_continue(self):
+        if self.signal_queue is not None:
+            while True:
+                try:
+                    sig = self.signal_queue.get(timeout=0.01)
+                except (Empty, IOError):
+                    break
+                if sig not in system.SIG_NAMES:
+                    self.logger.info("Ignoring unknown signal: %s", sig)
+                else:
+                    signame = system.SIG_NAMES.get(sig)
+                    if sig in EXIT_SIGNALS:
+                        self.logger.warn("Got signal %s. Stopping.", signame)
+                        self.stop(True)
+                        return False
+                    else:
+                        self.logger.debug('No handler for signal %s.', signame)
+        return True
+    
     def _run(self):
-        '''The run implementation which must be done by a derived class.'''
-        self._on_run()
         try:
             self.requestloop.start()
         finally:
             self.stop()
-    
\ No newline at end of file
