@@ -3,7 +3,6 @@ import os
 import sys
 import signal
 from multiprocessing import current_process
-from multiprocessing.queues import Empty
 
 import pulsar
 from pulsar.utils import system
@@ -12,22 +11,15 @@ from pulsar.utils.security import gen_unique_id
 from pulsar import HaltServer
 
 from .defer import itervalues, iteritems, multi_async, async
-from .actor import Actor, ACTOR_STATES,  ACTOR_TERMINATE_TIMEOUT,\
-                     ACTOR_STOPPING_LOOPS
+from .actor import Actor, ACTOR_STATES
 from .monitor import PoolMixin, _spawn_actor
 from .access import get_actor, set_actor
 from . import proxy
 
-try:    #pragma nocover
-    import queue
-except ImportError: #pragma nocover
-    import Queue as queue
-ThreadQueue = queue.Queue
-
 
 process_global = pulsar.process_global
 
-__all__ = ['arbiter', 'spawn', 'Arbiter', 'ThreadQueue']
+__all__ = ['arbiter', 'spawn', 'Arbiter']
 
 
 def arbiter(commands_set=None, **params):
@@ -53,7 +45,7 @@ the arbiter creates the actor it returns the proxy to the original caller.
 **Parameter kwargs**
 
 These optional parameters are:
-    * *actorcls* a custom :class:`Actor` subclass.
+    * *actor_class* a custom :class:`Actor` subclass.
     * *aid* the actor id
     * *commands_set* the set of :ref:`remote commands <api-remote_commands>`
       the :class:`Actor` can respond to.
@@ -91,7 +83,7 @@ It runs the main :class:`IOLoop` of your concurrent application.
 It is the equivalent of the gunicorn_ arbiter, the twisted_ reactor
 and the tornado_ eventloop.
 
-Users access the arbiter by the high level api::
+Users access the arbiter (in the arbiter process domain) by the high level api::
 
     import pulsar
 
@@ -103,8 +95,6 @@ Users access the arbiter by the high level api::
 '''
     pidfile = None
     restarted = False
-    EXIT_SIGNALS = (signal.SIGINT, signal.SIGTERM,
-                    signal.SIGABRT, system.SIGQUIT)
 
     ############################################################################
     # ARBITER HIGH LEVEL API
@@ -128,7 +118,7 @@ Users access the arbiter by the high level api::
         self.monitors[m.name] = m
         return m
 
-    def isprocess(self):
+    def is_process(self):
         return True
 
     def get_all_monitors(self):
@@ -161,33 +151,26 @@ Users access the arbiter by the high level api::
     ############################################################################
     # OVERRIDE ACTOR HOOKS
     ############################################################################
-    def on_init(self, **kwargs):
-        testing = kwargs.pop('__test_arbiter__',False)
-        if not testing:
-            if current_process().daemon:
-                raise pulsar.PulsarException(
-                        'Cannot create the arbiter in a daemon process')
-            if isinstance(get_actor(), self.__class__):
-                raise pulsar.PulsarException('Arbiter already created')
-            os.environ["SERVER_SOFTWARE"] = pulsar.SERVER_SOFTWARE
-        if self.cfg.daemon:
-            system.daemonize()
-        self.signal_queue = ThreadQueue()
-        return PoolMixin.on_init(self, **kwargs)
-
     def on_start(self):
+        if current_process().daemon:
+            raise pulsar.PulsarException(
+                    'Cannot create the arbiter in a daemon process')
+        os.environ["SERVER_SOFTWARE"] = pulsar.SERVER_SOFTWARE
+        if self.cfg.daemon: #pragma    nocover
+            system.daemonize()
         pidfile = self.cfg.pidfile
         if pidfile is not None:
             p = Pidfile(pidfile)
             p.create(self.pid)
             self.pidfile = p
+        PoolMixin.on_start(self)
 
-    def on_task(self):
-        '''Override the :class:`Actor.on_task` callback to perform the
-arbiter tasks at every iteration in the event loop.'''
+    def periodic_task(self):
+        # Arbiter periodic task
         if self.restarted:
             self.stop(exit_code=self.exit_code)
-        if not self._arbiter_task():
+        if self.can_continue() and self.running():
+            # managed actors job
             self.manage_actors()
             for m in list(itervalues(self.monitors)):
                 if m.started():
@@ -196,33 +179,18 @@ arbiter tasks at every iteration in the event loop.'''
                         self.monitors.pop(m.name)
                 else:
                     m.start()
-        try:
-            self.cfg.arbiter_task(self)
-        except:
-            pass
-
-    def manage_actor(self, actor):
-        '''If an actor failed to notify itself to the arbiter for more than
-the timeout. Stop the arbiter.'''
-        if self.running() and actor.notified:
-            gap = time() - actor.notified
-            if gap > actor.cfg.timeout:
-                if actor.stopping_loops < ACTOR_STOPPING_LOOPS:
-                    if not actor.stopping_loops:
-                        self.logger.info('Stopping %s. Timeout.', actor)
-                        self.send(actor, 'stop')
-                    actor.stopping_loops += 1
-                else:
-                    self.logger.warn('Terminating %s. Timeout.', actor)
-                    actor.terminate()
-                    actor.join(ACTOR_TERMINATE_TIMEOUT)
+            try:
+                self.cfg.arbiter_task(self)
+            except:
+                pass
+        self.ioloop.add_callback(self.periodic_task, False)
 
     @async()
     def on_stop(self):
         '''Stop the pools the message queue and remaining actors.'''
         if not self.ioloop.running():
             for pool in list(itervalues(self.monitors)):
-                pool.state = ACTOR_STATES.STOPPING
+                pool.state = ACTOR_STATES.INACTIVE
             self.state = ACTOR_STATES.RUN
             self.restarted = True
             self.ioloop.start()
@@ -233,43 +201,37 @@ the timeout. Stop the arbiter.'''
             yield self._close_message_queue()
     
     def on_exit(self):
-        self.logger.info("Shutting down server.")
         p = self.pidfile
         if p is not None:
             p.unlink()
-        if self.managed_actors:
+        if self.managed_actors or self.linked_actors:
             self.state = ACTOR_STATES.TERMINATE
-        if self.exit_code is not None:
+        self.logger.info("Bye.")
+        if self.exit_code:
             sys.exit(self.exit_code)
 
     ############################################################################
     # INTERNALS
     ############################################################################
-
     def _run(self):
-        """\
-        Initialize the arbiter. Start listening and set pidfile if needed.
-        """
-        self._on_run()
         try:
             self.cfg.when_ready(self)
         except:
             pass
         try:
-            self.ioloop.start()
+            self.requestloop.start()
         except HaltServer as e:
-            self._halt(reason=str(e), code=e.signal)
+            self._halt(reason=str(e))
         except (KeyboardInterrupt, SystemExit) as e:
             self._halt(e.__class__.__name__)
         except:
             self._halt(code=1)
 
     def _halt(self, reason=None, code=None):
-        #halt the arbiter. If there no signal ``sig`` it is an unexpected exit
         if not self.closed():
             if not reason:
-                reason = "Unhandled exception in main loop."
-                self.logger.critical(reason, exc_info=True)
+                self.logger.critical("Unhandled exception in main loop.",
+                                     exc_info=True)
             else:
                 self.logger.info(reason)
             self.stop(True, exit_code=code)
@@ -277,38 +239,3 @@ the timeout. Stop the arbiter.'''
     def _close_message_queue(self):
         return
 
-    def _arbiter_task(self):
-        '''Called by the Event loop to perform the signal handling from the
-signal queue'''
-        sig = None
-        while True:
-            try:
-                sig = self.signal_queue.get(timeout=0.01)
-            except (Empty, IOError):
-                break
-            if sig not in system.SIG_NAMES:
-                self.logger.info("Ignoring unknown signal: %s", sig)
-                sig = None
-            else:
-                signame = system.SIG_NAMES.get(sig)
-                if sig in self.EXIT_SIGNALS:
-                    raise HaltServer('Received Signal %s.' % signame, sig)
-                handler = getattr(self, "signal_%s" % signame.lower(), None)
-                if not handler:
-                    self.logger.debug('No Handle for signal "%s".', signame)
-                    sig = None
-                else:
-                    self.logger.info("Handling signal: %s", signame)
-                    sig = handler()
-        return sig
-
-    def signal(self, sig, frame=None):
-        signame = system.SIG_NAMES.get(sig, None)
-        if signame:
-            if self.ioloop.running():
-                self.logger.debug('Received and queueing signal %s.', signame)
-                self.signal_queue.put(sig)
-                self.ioloop.wake()
-        else:
-            self.logger.debug('Received unknown signal "%s". Skipping.', sig)
-
