@@ -31,6 +31,7 @@ __all__ = ['Deferred',
            'ispy3k',
            'NOT_DONE',
            'STOP_ON_FAILURE',
+           'EXIT_EXCEPTIONS',
            'CLEAR_ERRORS']
 
 ispy3k = sys.version_info >= (3, 0)
@@ -330,6 +331,10 @@ The implementation is very similar to the ``twisted.defer.Deferred`` object.
     ``True`` if the deferred was called. In this case the asynchronous result
     is ready and available in the :attr:`result`.
 
+.. attribute:: running
+
+    ``True`` if the deferred is running callbacks.
+
 """
     paused = 0
     _called = False
@@ -362,7 +367,7 @@ The function takes at most one argument, the result passed to the
 :meth:`callback` method. If the *errback* callable is provided it will
 be called when an exception occurs."""
         errback = errback if errback is not None else pass_through
-        if hasattr(callback,'__call__') and hasattr(errback,'__call__'):
+        if hasattr(callback, '__call__') and hasattr(errback, '__call__'):
             self._callbacks.append((callback, errback))
             self._run_callbacks()
         else:
@@ -374,7 +379,7 @@ be called when an exception occurs."""
         return self.add_callback(pass_through, errback)
 
     def addBoth(self, callback):
-        '''Equivalent to `self.add_callback(callback,callback)`.'''
+        '''Equivalent to `self.add_callback(callback, callback)`.'''
         return self.add_callback(callback, callback)
 
     def add_callback_args(self, callback, *args, **kwargs):
@@ -402,7 +407,6 @@ this point, :meth:`add_callback` will run the *callbacks* immediately.
     def result_or_self(self):
         '''Obtain the result if available, otherwise it returns self.'''
         return self.result if self._called and not self.paused else self
-        #return self.result if self._called else self
 
     ##################################################    INTERNAL METHODS
     def _run_callbacks(self):
@@ -612,14 +616,8 @@ both ``list`` and ``dict`` types.'''
         if is_generalised_generator(value):
             value = list(value)
         value = maybe_async(value)
-        if is_async(value):
-            self._add_deferred(key, value)
-        else:
-            # if an instance of an iterable create multiple objects
-            if isinstance(value, (dict, list, tuple, set, frozenset)):
-                value = self._make(value)
-                if is_async(value):
-                    self._add_deferred(key, value)
+        if isinstance(value, (dict, list, tuple, set, frozenset)):
+            value = self._make(value)    
         if not is_async(value) and self.handle_value:
             try:
                 val = self.handle_value(value)
@@ -629,6 +627,9 @@ both ``list`` and ``dict`` types.'''
                 if val is not value:
                     return self._add(key, val)
         self._setitem(key, value)
+        # add callback if an asynchronous value
+        if is_async(value):
+            self._add_deferred(key, value)
 
     def _make(self, value):
         md = self.__class__(value, fireOnOneErrback=self.fireOnOneErrback,
