@@ -23,7 +23,7 @@ __all__ = ['IOLoop', 'PeriodicCallback', 'loop_timeout']
 LOGGER = logging.getLogger('pulsar.eventloop')
 
 def file_descriptor(fd):
-    if hasattr(fd,'fileno'):
+    if hasattr(fd, 'fileno'):
         return fd.fileno()
     else:
         return fd
@@ -40,18 +40,18 @@ class LoopGuard(object):
 
     def __enter__(self):
         loop = self.loop
-        loop.logger.debug("Starting event loop")
+        loop.logger.subdebug("Starting event loop")
         loop._running = True
         if not loop._started:
             loop._started = time.time()
         setid(loop)
-        loop._on_exit = Deferred()
+        loop._on_exit = Deferred(description='IOloop.on_exit')
         return self
 
     def __exit__(self, type, value, traceback):
         loop = self.loop
         loop._running = False
-        loop.logger.debug('Exiting event loop')
+        loop.logger.subdebug('Exiting event loop')
         loop._on_exit.callback(loop)
 
 
@@ -65,9 +65,21 @@ A level-triggered I/O event loop adapted from tornado.
     it has an ``epoll`` like interface. Pulsar ships with an additional
     I/O implementation based on distributed queue :class:`IOQueue`.
 
-.. attribute:: num_lumps
+**ATTRIBUTES**
 
-    total number of loops
+.. attribute:: _impl
+
+    The IO implementation
+
+.. attribute:: cpubound
+
+    If ``True`` this is a CPU bound event loop, otherwise it is an I/O
+    event loop. CPU bound loops can block the loop for considerable amount
+    of time.
+        
+.. attribute:: num_loops
+
+    Total number of loops
 
 .. attribute:: poll_timeout
 
@@ -78,11 +90,20 @@ A level-triggered I/O event loop adapted from tornado.
 .. attribute:: tid
 
     The thread id where the eventloop is running
+    
+.. attribute:: tasks
+
+    A list of callables to be executed at each iteration of the event loop.
+    Task can be added and deleted via the :meth:`add_task` and
+    :meth:`remove_task`. Extra care must be taken when adding tasks to
+    I/O event loops. These tasks should be fast to perform and not block.
+
+**METHODS**
 """
     # Never use an infinite timeout here - it can stall epoll
     poll_timeout = 0.5
 
-    def __init__(self, io=None, logger=None, poll_timeout=None, ready=True):
+    def __init__(self, io=None, logger=None, poll_timeout=None):
         self._impl = io or IOpoll()
         self.poll_timeout = poll_timeout if poll_timeout else self.poll_timeout
         self.logger = logger or LOGGER
@@ -92,13 +113,11 @@ A level-triggered I/O event loop adapted from tornado.
         self._events = {}
         self._callbacks = []
         self._timeouts = []
-        self._loop_tasks = WeakList()
         self._started = None
         self._running = False
         self.num_loops = 0
         self._waker = getattr(self._impl, 'waker', Waker)()
         self._on_exit = None
-        self.ready = ready
         self.add_handler(self._waker,
                          lambda fd, events: self._waker.consume(),
                          self.READ)
@@ -107,19 +126,6 @@ A level-triggered I/O event loop adapted from tornado.
     def cpubound(self):
         return getattr(self._impl, 'cpubound', False)
 
-    def add_loop_task(self, task):
-        '''Add a callable object to the list of tasks which are
-executed at each iteration in the event loop.'''
-        self._loop_tasks.append(task)
-
-    def remove_loop_task(self, task):
-        '''Remove the task from the list of tasks
-executed at each iteration in the event loop.'''
-        try:
-            return self._loop_tasks.remove(task)
-        except ValueError:
-            pass
-
     def add_handler(self, fd, handler, events):
         """Registers the given *handler* to receive the given events for the
 file descriptor *fd*.
@@ -234,9 +240,6 @@ will make the loop stop after the current event iteration completes."""
                 poll_timeout = self.poll_timeout
                 self.num_loops += 1
                 _run_callback = self._run_callback
-                # Loop task first
-                for task in self._loop_tasks:
-                    _run_callback(task, 'loop task')
                 # Prevent IO event starvation by delaying new callbacks
                 # to the next iteration of the event loop.
                 callbacks = self._callbacks
@@ -255,9 +258,6 @@ will make the loop stop after the current event iteration completes."""
                 # A chance to exit
                 if not self._running:
                     break
-                # Not ready to poll
-                if not self.ready:
-                    continue
                 try:
                     event_pairs = self._impl.poll(poll_timeout)
                 except Exception as e:
