@@ -501,7 +501,8 @@ via the ``connect`` function).
     :class:`ClientSocketHandler`.
 '''
     parser_class = EchoParser
-    def __init__(self, socket, address, parser_class=None, timeout=None):
+    def __init__(self, socket, address, parser_class=None, timeout=None,
+                  read_timeout=None):
         '''Create a client or client-connection socket. A parser class
 is required in order to use :class:`SocketClient`.
 
@@ -509,10 +510,14 @@ is required in order to use :class:`SocketClient`.
 :parameter address: The address of the remote client/server
 :parameter parser_class: A class used for parsing messages.
 :parameter timeout: A timeout in seconds for the socket. Same rules as
-    the ``socket.settimeout`` method in the standard library.
+    the ``socket.settimeout`` method in the standard library. A value of 0
+    indicates an asynchronous socket.
+:parameter read_timeout: A timeout in seconds for asynchronous operations. This
+    value is only used when *timeout* is 0 in the constructor
+    of a :class:`AsyncIOStream`.
 '''
         self._socket_timeout = get_socket_timeout(timeout)
-        self._set_socket(socket)
+        self._set_socket(socket, read_timeout)
         self.remote_address = address
         parser_class = parser_class or self.parser_class
         self.parser = parser_class()
@@ -522,7 +527,7 @@ is required in order to use :class:`SocketClient`.
         return str(self.remote_address)
     __str__ = __repr__
 
-    def _set_socket(self, sock):
+    def _set_socket(self, sock, read_timeout=None):
         if not isinstance(sock, AsyncIOStream):
             if self._socket_timeout == 0:
                 sock = AsyncIOStream(sock)
@@ -533,6 +538,7 @@ is required in order to use :class:`SocketClient`.
         if self.async:
             close_callback = Deferred().add_callback(self.close)
             self.sock.set_close_callback(close_callback)
+            self.read_timeout = read_timeout
             
     def _get_read_timeout(self):
         if self.async:
@@ -544,7 +550,7 @@ is required in order to use :class:`SocketClient`.
             self.sock._read_callback_timeout = value
         else:
             self._socket_timeout = value
-            sock.settimeout(self._socket_timeout)
+            self.sock.settimeout(self._socket_timeout)
     read_timeout = property(_get_read_timeout, _set_read_timeout) 
 
 
@@ -558,9 +564,10 @@ reading from the same socket connection.'''
         self.processing = False
         
     @classmethod
-    def connect(cls, address, parser_class=None, timeout=None):
+    def connect(cls, address, **kwargs):
+        '''Create a new :class:`ClientSocket` connected at *address*.'''
         sock = create_connection(address)
-        return cls(sock, address, parser_class=parser_class, timeout=timeout)
+        return cls(sock, address, **kwargs)
 
     def send(self, data):
         '''Send data to remote server'''
@@ -646,13 +653,18 @@ class ReconnectingClient(Client):
 
 
 class AsyncResponse(object):
-    '''An asynchronous response is created once a connection has produced
-finished data from a read operation. Instances of this class are iterable over
+    '''An asynchronous server response is created once an
+:class:`AsyncConnection` has available parsed data from a read operation.
+Instances of this class are iterable over
 chunk of data to send back to the remote client.
 
 .. attribute:: connection
 
     The :class:`AsyncConnection` for this response
+    
+.. attribute:: server
+
+    The :class:`AsyncSocketServer` for this response
 
 .. attribute:: parsed_data
 
@@ -675,6 +687,7 @@ chunk of data to send back to the remote client.
         return self.connection.sock
 
     def __iter__(self):
+        # by default it echos the client message
         yield self.parsed_data
 
 
@@ -685,27 +698,33 @@ A connection can handle several request/responses until it is closed.
 
 .. attribute:: server
 
-    The class :class:`AsyncSocketServer` which created the connection
+    The class :class:`AsyncSocketServer` which created this
+    :class:`AsyncConnection`.
 
+.. attribute:: response_class
+
+    Class or callable for building an :class:`AsyncResponse` object. It is
+    initialised by the :class:`AsyncSocketServer.response_class` but it can be
+    changed at runtime when upgrading connections to new protocols. An example
+    is the websocket protocol.
 '''
-    response_class = AsyncResponse
-    '''Class or callable for building an :class:`AsyncResponse` object.'''
-
     def __init__(self, sock, address, server, timeout=None):
         if not isinstance(sock, AsyncIOStream):
             sock = AsyncIOStream(sock, timeout=server.timeout)
         super(AsyncConnection, self).__init__(sock, address,
                                               server.parser_class)
         self.server = server
+        self.response_class = server.response_class
         server.connections.add(self)
         self.handle().add_errback(self.close)
 
-    @async(max_errors=1, description='Async client connection generator')
+    @async(max_errors=1, description='Asynchronous client connection generator')
     def handle(self):
         while not self.closed:
             outcome = self.sock.read()
             yield outcome
-            # if we sare here it means no errors occurred so far
+            # if we are here it means no errors occurred so far and
+            # data is available to process
             self.time_last = time.time()
             buffer = self.buffer
             buffer.extend(outcome.result)
@@ -715,7 +734,7 @@ A connection can handle several request/responses until it is closed.
                 parsed_data = self.request_data()
                 if parsed_data:
                     self.received += 1
-                    # The response is iterable (same as WSGI response)
+                    # The response is an iterable (same as WSGI response)
                     for data in self.response_class(self, parsed_data):
                         if data:
                             yield self.write(data)
@@ -760,11 +779,12 @@ more data in the buffer is required.'''
 
 
 class AsyncSocketServer(BaseSocketHandler):
-    '''A :class:`BaseSocketHandler` for asynchronous socket servers.
+    '''A :class:`BaseSocketHandler` for asynchronous servers which listen
+for requests on a socket.
 
 .. attribute:: actor
 
-    The :class:`Actor` powering this :class:`AsyncSocketServer`.
+    The :class:`Actor` running this :class:`AsyncSocketServer`.
 
 .. attribute:: ioloop
 
@@ -787,25 +807,38 @@ class AsyncSocketServer(BaseSocketHandler):
 .. attribute:: timeout
 
     The timeout when reading data in an asynchronous way.
+    
+.. attribute:: response_class
+
+    A subclass of :class:`AsyncResponse` for handling responses to clients
+    once data has been received and processed.
 '''
     thread = None
     _started = False
     connection_class = AsyncConnection
     parser_class = EchoParser
+    response_class = AsyncResponse
+    
     def __init__(self, actor, socket, parser_class=None, onthread=False,
-                 connection_class=None, timeout=None):
+                  connection_class=None, response_class=None, timeout=None):
         self.actor = actor
         self.parser_class = parser_class or self.parser_class
         self.connection_class = connection_class or self.connection_class
+        self.response_class = response_class or self.response_class
         self.sock = wrap_socket(socket)
         self.connections = set()
-        self.onthread = onthread
         self.timeout = timeout
         self.on_connection_callbacks = []
-        # If the actor has a ioqueue (CPU bound actor) we create a new ioloop
-        if self.onthread:
-            self.__ioloop = IOLoop(logger=actor.logger)
-
+        if onthread:
+            # Create a pulsar thread and starts it
+            self.__ioloop = IOLoop()
+            self.thread = PulsarThread(name=self.name, target=self._run)
+            self.thread.actor = actor
+            actor.requestloop.add_callback(lambda: self.thread.start())
+        else:
+            self.__ioloop = actor.requestloop
+            actor.requestloop.add_callback(self._run)
+        
     @classmethod
     def make(cls, actor=None, bind=None, backlog=None, **kwargs):
         if actor is None:
@@ -822,74 +855,49 @@ class AsyncSocketServer(BaseSocketHandler):
         return cls(actor, socket, **kwargs)
 
     @property
-    def name(self):
-        return '%s %s' % (self.actor, self.address)
+    def onthread(self):
+        return self.thread is not None
     
     @property
-    def started(self):
-        return self._started
+    def name(self):
+        return '%s %s' % (self.__class__.__name__, self.address)
 
     def __repr__(self):
         return self.name
     __str__ = __repr__
 
-    def start(self):
-        self.actor.requestloop.add_callback(self._start)
-        return self
-
     @property
     def ioloop(self):
-        return self.__ioloop if self.onthread else self.actor_ioloop()
+        return self.__ioloop
 
     @property
     def active_connections(self):
         return len(self.connections)
-    
-    def actor_ioloop(self):
-        return self.actor.ioloop
-
-    def on_start(self):
-        '''callback just before the event loop starts.'''
-        pass
-
-    def shut_down(self):
-        pass
 
     def quit_connections(self):
         for c in list(self.connections):
             c.close()
             
     def on_close(self, failure=None):
-        self.shut_down()
         self.quit_connections()
         self.ioloop.remove_handler(self)
         if self.onthread:
-            self.ioloop.stop()
+            self.__ioloop.stop()
             # We join the thread
             if current_thread() != self.thread:
                 self.thread.join()
 
     ############################################################## INTERNALS
-    def _start(self):
-        # If onthread is true we start the event loop on a
-        # separate thread.
-        if self.onthread:
-            if self.thread and self.thread.is_alive():
-                raise RunTimeError('Cannot start socket server. '\
-                                   'It has already started')
-            self.thread = PulsarThread(name=self.name, target=self._run)
-            self.thread.start()
-        if not self._started:
-            self._started = True
-            self.ioloop.add_handler(self,
-                                    self.new_connection,
-                                    self.ioloop.READ)
-            return self.on_start()
-
     def _run(self):
+        # add new_connection READ handler to the eventloop and starts the
+        # eventloop if it was not already started.
+        self.actor.logger.debug('Registering %s with event loop.', self)
+        self.ioloop.add_handler(self,
+                                self._on_connection,
+                                self.ioloop.READ)
         self.ioloop.start()
 
-    def new_connection(self, fd, events):
+    def _on_connection(self, fd, events):
         '''Called when a new connection is available.'''
         # obtain the client connection
         for callback in self.on_connection_callbacks:
