@@ -35,7 +35,7 @@ otherwise a queue is used.'''
         if actor.is_monitor():
             return MonitorMailbox(actor)
         else:
-            return Mailbox.make(actor).start()
+            return Mailbox.make(actor)
 
 def actorid(actor):
     return actor.aid if hasattr(actor, 'aid') else actor
@@ -196,7 +196,6 @@ from a remote :class:`Actor`.
 An instance of this class is created when a new connection is made
 with a :class:`Mailbox`.'''
     authenticated = False
-    response_class = MailboxResponse
 
 
 class Mailbox(AsyncSocketServer):
@@ -206,43 +205,21 @@ creates its own :class:`IOLoop` which runs on a separate thread
 of execution.'''
     parser_class = MessageParser
     connection_class = MailboxConnection
+    response_class = MailboxResponse
 
     @classmethod
     def make(cls, actor, backlog=64):
-        return super(Mailbox, cls).make(actor=actor, backlog=backlog,
-                                        onthread=actor.cpubound,
-                                        timeout=None)
+        server = super(Mailbox, cls).make(actor=actor, backlog=backlog,
+                                          onthread=actor.cpubound,
+                                          timeout=None)
+        if not server.actor.is_arbiter():
+            server.ioloop.add_callback(server.send_mailbox_address)
+        return server
 
-    @property
-    def name(self):
-        return '%s mailbox %s:%s' %\
-             (self.actor, self.address[0], self.address[1])
-
-    @property
-    def cpubound(self):
-        return self.actor.cpubound
-
-    def actor_ioloop(self):
-        return self.actor.requestloop
-    
-    def shut_down(self):
-        self.unregister(self.actor)
-
-    def on_start(self):
+    def send_mailbox_address(self):
         actor = self.actor
-        self.register(actor)
-        if not actor.is_arbiter():
-            # The actor is not the arbiter. We need to register this mailbox
-            # with the actor monitor so that it can receive messages from it
-            return actor.send(actor.monitor, 'mailbox_address', self.address)\
-                        .add_callback(actor.link_actor)
-
-    def register(self, actor):
-        self.ioloop.add_loop_task(actor)
-
-    def unregister(self, actor):
-        if not self.ioloop.remove_loop_task(actor):
-            LOGGER.warn('"%s" could not be removed from eventloop', actor)
+        return actor.send(actor.monitor, 'mailbox_address', self.address)\
+                    .add_callback(actor.link_actor)
 
 
 class MonitorMailbox(object):
@@ -250,13 +227,7 @@ class MonitorMailbox(object):
 arbiter inbox.'''
     active_connections = 0
     def __init__(self, actor):
-        self.actor = actor
         self.mailbox = actor.arbiter.mailbox
-        self.mailbox.register(self.actor)
-
-    @property
-    def cpubound(self):
-        return self.actor.cpubound
 
     @property
     def address(self):
@@ -266,11 +237,11 @@ arbiter inbox.'''
     def ioloop(self):
         return self.mailbox.ioloop
 
-    def start(self):
+    def _run(self):
         pass
-
+    
     def close(self):
-        self.mailbox.unregister(self.actor)
+        pass
 
 
 class QueueWaker(object):
