@@ -1,11 +1,13 @@
 import os
 import sys
-import time
+from time import time
 
 import pulsar
+from pulsar.utils.structures import AttributeDictionary
 
 from . import proxy
-from .actor import Actor, ACTOR_STATES, ACTOR_TERMINATE_TIMEOUT
+from .actor import Actor, ACTOR_STATES, ACTOR_TERMINATE_TIMEOUT,\
+                     ACTOR_STOPPING_LOOPS
 from .eventloop import setid
 from .concurrency import concurrency
 from .defer import async, iteritems, itervalues, range, NOT_DONE
@@ -19,8 +21,8 @@ def _spawn_actor(cls, commands_set=None, monitor=None, cfg=None, name=None,
                  aid=None, **kw):
     # Internal function which spawns a new Actor and return its
     # ActorProxyMonitor.
+    # *cls* is the Actor class
     # *monitor* can be either the ariber or a monitor
-    # *actorcls* is the Actor class
     kind = None
     if issubclass(cls, PoolMixin):
         kind = 'monitor'
@@ -90,18 +92,13 @@ during its life time.
 
     Default: :class:`Actor`'''
 
-    def on_init(self, actor_class=None, **kwargs):
+    def on_start(self):
         self.spawning_actors = {}
         self.managed_actors = {}
-        self.actor_class = actor_class or self.actor_class
-        return kwargs
+        self.actor_class = self.params.pop('actor_class') or self.actor_class
 
-    def __call__(self):
-        if self.running():
-            self.on_task()
-
-    def ready(self):
-        return True
+    def active(self):
+        return self.running()
 
     def spawn(self, actor_class=None, linked_actors=None, montitor=None,
               **params):
@@ -135,20 +132,20 @@ spawn method when creating new actors.'''
 :parameter stop: if ``True`` stops all alive actor.
 :parameter manage: if ``True`` it checks if alive actors are still responsive.
 '''
-        ACTORS = self.managed_actors
+        MANAGED = self.managed_actors
         SPAWNING = self.spawning_actors
         LINKED = self.linked_actors
         alive = 0
-        items = list(iteritems(ACTORS))
-        # WHEN TERMINATING OR STOPPING WE INCLUDE THE ACTORS WHICH ARE
-        # SPAWNING
+        ACTORS = list(iteritems(MANAGED))
+        # WHEN TERMINATING OR STOPPING WE INCLUDE THE ACTORS WHICH ARE SPAWNING
         shutting_down = terminate or stop
         if shutting_down:
-            items.extend(iteritems(SPAWNING))
-        for aid, actor in items:
+            ACTORS.extend(iteritems(SPAWNING))
+        # Loop over MANAGED ACTORS PROXY MONITORS
+        for aid, actor in ACTORS:
             if not actor.is_alive():
                 actor.join(ACTOR_TERMINATE_TIMEOUT)
-                ACTORS.pop(aid, None)
+                MANAGED.pop(aid, None)
                 SPAWNING.pop(aid, None)
                 LINKED.pop(aid, None)
             else:
@@ -166,8 +163,21 @@ spawn method when creating new actors.'''
         return alive
 
     def manage_actor(self, actor):
-        '''This function is overwritten by the arbiter'''
-        pass
+        '''If an actor failed to notify itself to the arbiter for more than
+the timeout. Stop the arbiter.'''
+        stopping_loops = actor.stopping_loops
+        if self.running() and (stopping_loops or actor.notified):
+            gap = time() - actor.notified
+            if gap > actor.cfg.timeout or stopping_loops:
+                if stopping_loops < ACTOR_STOPPING_LOOPS:
+                    if not stopping_loops:
+                        self.logger.info('Stopping %s. Timeout.', actor)
+                        self.send(actor, 'stop')
+                    actor.stopping_loops += 1
+                else:
+                    self.logger.warn('Terminating %s. Timeout.', actor)
+                    actor.terminate()
+                    actor.join(ACTOR_TERMINATE_TIMEOUT)
 
     def spawn_actors(self):
         '''Spawn new actors if needed. If the :class:`PoolMixin` is spawning
@@ -195,9 +205,11 @@ as required."""
 
     def link_actor(self, proxy, address):
         # Override the link_actor from Actor class
+        self.logger.debug('Registering %s address %s', proxy, address)
         if proxy.aid not in self.spawning_actors:
             raise RuntimeError('Could not retrieve proxy %s.' % proxy)
         proxy_monitor = self.spawning_actors.pop(proxy.aid)
+        proxy_monitor.info['last_notified'] = time()
         on_address = proxy_monitor.on_address
         delattr(proxy_monitor,'on_address')
         try:
@@ -218,13 +230,13 @@ as required."""
     @async()
     def close_actors(self):
         '''Close all managed :class:`Actor`.'''
-        start = time.time()
+        start = time()
         # Stop all of them
         to_stop = self.manage_actors(stop=True)
         while to_stop:
             yield NOT_DONE
             to_stop = self.manage_actors(manage=False)
-            dt = time.time() - start
+            dt = time() - start
             if dt > self.CLOSE_TIMEOUT:
                 self.logger.warn('Cannot stop %s actors.', to_stop)
                 to_stop = self.manage_actors(terminate=True)
@@ -269,7 +281,7 @@ Monitors with distributed queues manage CPU-bound :class:`Actors`.
     def cpubound(self):
         return False
 
-    def isprocess(self):
+    def is_process(self):
         return False
 
     def is_monitor(self):
@@ -281,8 +293,7 @@ Monitors with distributed queues manage CPU-bound :class:`Actors`.
 By default it does nothing.'''
         pass
 
-    # HOOKS
-    def on_task(self):
+    def periodic_task(self):
         '''Overrides the :meth:`Actor.on_task`
 :ref:`actor callback <actor-callbacks>` to perform
 the monitor :class:`IOLoop` tasks, which are:
@@ -301,25 +312,19 @@ The implementation goes as following:
 
 Users shouldn't need to override this method, but use
 :meth:`Monitor.monitor_task` instead.'''
-        self.manage_actors()
-        self.spawn_actors()
-        self.stop_actors()
-        return self.monitor_task()
+        if self.running():
+            self.manage_actors()
+            self.spawn_actors()
+            self.stop_actors()
+            self.monitor_task()
+        self.ioloop.add_callback(self.periodic_task, False)
 
+    # HOOKS
     def on_stop(self):
         '''Overrides the :meth:`Actor.on_stop`
 :ref:`actor callback <actor-callbacks>` to stop managed actors.'''
         return self.close_actors()
 
-    # OVERRIDES INTERNALS
-    def _run(self):
-        self.requestloop = self.arbiter.requestloop
-        self.mailbox = mailbox(self)
-        self.monitors = self.arbiter.monitors
-        setid(self)
-        self.state = ACTOR_STATES.RUN
-        self.on_start()
-
     @property
     def multithread(self):
         return self.cfg.concurrency == 'thread'
@@ -343,13 +348,15 @@ Users shouldn't need to override this method, but use
             return actor.proxy.stop()
 
     def info(self):
-        tq = self.ioqueue
         data = {'actor': {'actor_class':self.actor_class.__name__,
                           'concurrency':self.cfg.concurrency,
                           'name':self.name,
                           'age':self.impl.age,
-                          'workers': len(self.managed_actors)},
-                'workers': [a.info for a in itervalues(self.managed_actors)]}
+                          'workers': len(self.managed_actors)}}
+        if not self.started():
+            return data
+        data['workers'] = [a.info for a in itervalues(self.managed_actors)]
+        tq = self.ioqueue
         if tq is not None:
             if isinstance(tq, Queue):
                 tqs = 'multiprocessing.Queue'
@@ -369,3 +376,11 @@ Users shouldn't need to override this method, but use
             a = self.arbiter.get_actor(aid)
         return a
 
+    # OVERRIDES INTERNALS
+    def _setup_ioloop(self):
+        self.requestloop = self.arbiter.requestloop
+        self.mailbox = mailbox(self)
+        self.monitors = self.arbiter.monitors
+        
+    def _run(self):
+        pass
