@@ -1,5 +1,4 @@
-'''Operative system specific functions and classes
-'''
+'''Operative system specific functions and classes.'''
 from .runtime import Platform
 
 platform = Platform()
@@ -7,15 +6,15 @@ seconds = platform.seconds
 
 from .base import *
 
-if platform.type == 'posix':
+if platform.type == 'posix':    #pragma nocover
     from .posixsystem import *
-elif platform.type == 'win':
+elif platform.type == 'win':    #pragma nocover
     from .windowssystem import *
 
 
 try:
     import psutil
-except ImportError:
+except ImportError:    #pragma    nocover
     psutil = None
     
 memory_symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
@@ -23,13 +22,15 @@ memory_size = dict(((s,1 << (i+1)*10) for i,s in enumerate(memory_symbols)))
 
 
 def convert_bytes(b):
+    '''Convert a number of bytes into a human readable memory usage, bytes,
+kilo, mega, giga, tera, peta, exa, zetta, yotta'''
     if b is None:
         return '#NA'
     for s in reversed(memory_symbols):
         if b >= memory_size[s]:
             value = float(b) / memory_size[s]
             return '%.1f%sB' % (value, s)
-    return "%sB" % n
+    return "%sB" % b
     
     
 def system_info(pid):
@@ -39,7 +40,7 @@ it returns an empty dictionary.
 
 .. _psutil: http://code.google.com/p/psutil/
 '''
-    if psutil is None:
+    if psutil is None:  #pragma    nocover
         return {}
     p = psutil.Process(pid)
     mem = p.get_memory_info()
